/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"          // Module interface definitions
#include "AEEAppGen.h"          // Applet interface definitions
#include "AEEShell.h"           // Shell interface definitions
#include "AEEDisp.h"            // Display interface defintions
#include "developkamja.bid"		// applet class ID
#include "developkamja_res.h"	// resource definitions
#include "AEEFile.h"		//	AEEFile Services
#include "AEEMenu.h"
#include "AEEText.h"
#include "AEEStdLib.h"
#include "AEEComdef.h"

//
// This definition must be enabled in order for you to define custom color 
// settings for the app.  
//
#define CONFIGURE_COLORS

#if defined(CONFIGURE_COLORS)

#define MCM_BACK     RGB_WHITE
#define MCM_TEXT     RGB_BLACK
#define MCM_SELBACK  LIGHT_GREY
#define MCM_SELTEXT  RGB_BLACK
#define MCM_FRAME    RGB_WHITE

#define SKC_BACK     DK_BLUE
#define SKC_TEXT     RGB_WHITE
#define SKC_SELTEXT  RGB_WHITE
#define SKC_FRAME    LIGHT_GREY

#endif
//
// A few helpful color values...
//
#define VERY_LT_BLUE			MAKE_RGB(128,128,255)
#define BLUE					MAKE_RGB(0,128,255)
#define DK_BLUE					MAKE_RGB(23,0,94)
#define DARK_GREY				MAKE_RGB(0x80, 0x80, 0x80) // dark grey, intensity == 0.33
#define LIGHT_GREY				MAKE_RGB(0xc0, 0xc0, 0xc0) // light grey, intensity == 0.67
// 
// Main Applet Structure
//// subject : icon view menu
//
#define APP_RES_FILE          "DevelopKamja.bar"

typedef struct _CDevelopKamja
{
	AEEApplet         a;
	
	IMenuCtl *        m_pMenu;          // the main icon view menu
	IMenuCtl *		  m_pSKMenu;			// text control
	
	// default images
	IImage      *     m_pKamja;
	IImage      *     m_pJahyen;
	AEERect			m_rc;
	uint16            m_cy;

} CDevelopKamja;

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static boolean	DevelopKamja_HandleEvent(CDevelopKamja * pme, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static void		DevelopKamja_Free(CDevelopKamja* pme);
static void		DevelopKamja_Start(CDevelopKamja* pme);
/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */
/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to InitAppData(). This is done as follows:
   InitAppData() is called to initialize AppletData instance. It is app developers 
   responsibility to fill-in app data initialization code of InitAppData(). App developer
   is also responsible to release memory allocated for data contained in AppletData -- this can be
   done in FreeAppData().

PROTOTYPE:
	int AEEAppCreateInstance(AEECLSID clsID, IShell* pIShell, IModule* pIModule, 
										  IApplet** ppApplet)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppApplet: [out]: On return, *ppApplet must point to a valid AEEApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was successful
  EFAILED: If the app does not need to be loaded or if errors occurred in AEEApplet_New().
  If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
	CDevelopKamja *pme;
	AEEMenuColors mclr;
	AEEItemStyle	n,s;
	AEEDeviceInfo  devInfo;
	*ppObj = NULL;
		
	if(ClsId == AEECLSID_DEVELOPKAMJA){
      if(!AEEApplet_New(sizeof(CDevelopKamja), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)DevelopKamja_HandleEvent,(PFNFREEAPPDATA)DevelopKamja_Free))
      {
         return (ENOMEMORY);
      }
	}
	pme = (CDevelopKamja*)*ppObj;
	
	if(ISHELL_CreateInstance(pIShell, AEECLSID_SOFTKEYCTL, (void**)(&pme->m_pSKMenu)))
	{
		IAPPLET_Release((IApplet*)pme);
		return (EFAILED);
	}
	
	if(ISHELL_CreateInstance(pIShell, AEECLSID_ICONVIEWCTL, (void**)(&pme->m_pMenu)))
	{
		IAPPLET_Release((IApplet*)pme);
		return (EFAILED);
	}
	
	IMENUCTL_SetProperties(pme->m_pMenu, MP_ICON_SINGLE_FRAME|MP_NO_ARROWS|MP_ICON_ANIMATED|MP_ICON_DELAY_IMAGE);
	IMENUCTL_SetProperties(pme->m_pSKMenu, MP_MAXSOFTKEYITEMS);

	ISHELL_GetDeviceInfo(pIShell,&devInfo);
	
	if(devInfo.nColorDepth >= 8) 
	{
	   mclr.wMask = (MC_BACK | MC_TEXT | MC_SEL_BACK | MC_SEL_TEXT | MC_FRAME);

	   mclr.cBack     = MCM_BACK;
	   mclr.cText     = MCM_TEXT;
	   mclr.cSelBack  = MCM_SELBACK;
	   mclr.cSelText  = MCM_SELTEXT;
	   mclr.cFrame    = MCM_FRAME;

	   IMENUCTL_SetColors(pme->m_pMenu,&mclr);

	   mclr.cBack     = SKC_BACK;
	   mclr.cText     = SKC_TEXT;
	   mclr.cSelText  = SKC_SELTEXT;
	   mclr.cFrame    = SKC_FRAME;
	   IMENUCTL_SetColors(pme->m_pSKMenu,&mclr);
	}

	ISHELL_GetItemStyle(pme->a.m_pIShell,AEE_IT_ICONVIEW,&n,&s);
	n.yOffset = s.yOffset = 0;
	IMENUCTL_SetStyle(pme->m_pMenu,&n,&s);

	// Setup the desired tabbed icon view settings to make it look like
	// a tabbed dialog...

	ISHELL_GetItemStyle(pme->a.m_pIShell,AEE_IT_SOFTKEY,&n,&s);
	n.ft = AEE_FT_TAB_BOTTOM;
	n.xOffset =  s.xOffset = 0;
	n.yOffset = s.yOffset = 1;
	s.ft = AEE_FT_TAB_BOTTOM_SEL;
	IMENUCTL_SetStyle(pme->m_pSKMenu,&n,&s);
	return(AEE_SUCCESS);
}

/*===========================================================================


===========================================================================*/
static boolean DevelopKamja_HandleEvent(CDevelopKamja * pme, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
	uint16 nSel;

   switch (eCode) 
	{
      case EVT_APP_START:                        
			DevelopKamja_Start(pme);
			return TRUE;
	  case EVT_APP_STOP:
			return TRUE;
	  case EVT_KEY: 
			switch (wParam)
			{
			   default:
			      if (IMENUCTL_HandleEvent(pme->m_pMenu, EVT_KEY, wParam, 0))
				  {
						nSel = IMENUCTL_GetSel(pme->m_pMenu);
						if(nSel != IMENUCTL_GetSel(pme->m_pSKMenu))
							IMENUCTL_SetSel(pme->m_pSKMenu, nSel);
				  }
			      return TRUE;
			}
		case EVT_COMMAND:
  		    switch(wParam)
			{
				case IDS_KAMJA_TITLE:
	        			return TRUE;	
                case IDS_JAHYEN_TITLE:
	        			return TRUE;
				default:
					return FALSE;
			}
      default:
			break;
   }

   return FALSE;
}

/*===========================================================================
FUNCTION: CDevelopKamja_Start

DESCRIPTION
		Called by HandleEvent on EVT_APP_START

PARAMETERS
		pme : the app mgr
===========================================================================*/
static void DevelopKamja_Start(CDevelopKamja* pme)
{
	IShell *       pShell = pme->a.m_pIShell;
	AEERect			rc,rcs;
	AEEDeviceInfo	di;
	CtlAddItem		pai;
	int				cy, ccy, m_cxWidth, m_cyHeight;
	int				cLineHeight;
	char szResFile[] = APP_RES_FILE;


	ISHELL_GetDeviceInfo(pme->a.m_pIShell, &di);
	cy = di.cyScreen;
	
    pai.pText  = NULL;        // Text 
    pai.pImage = NULL;        // Image 

    pai.pszResImage = szResFile;   // Resource for Image
    pai.pszResText  = szResFile;   // Resource for Text

	// Use the default font for all item text
    pai.wFont = 0;         // 0 - Default

    pai.wText   = IDS_KAMJA_TITLE; 
    pai.wImage  = IDB_KAMJA_ICON;
    pai.wItemID = IDS_KAMJA_TITLE;
	IMENUCTL_AddItemEx(pme->m_pMenu, &pai);
	
	pai.pszResText	= NULL;
	pai.wText   = 0; 
	pai.wImage  = IDB_KAMJA_THUMB;
    pai.wItemID = IDS_KAMJA_TITLE;
	
    IMENUCTL_AddItemEx(pme->m_pSKMenu, &pai);
	
	pai.pszResText	= szResFile;
	pai.wText   = IDS_JAHYEN_TITLE; 
    pai.wImage  = IDB_JAHYEN_ICON;
    pai.wItemID = IDS_JAHYEN_TITLE;
	
	IMENUCTL_AddItemEx(pme->m_pMenu, &pai);

	pai.pszResText	= NULL;
	pai.wText   = 0; 
    pai.wImage  = IDB_JAHYEN_THUMB;
    pai.wItemID = IDS_JAHYEN_TITLE;

	IMENUCTL_AddItemEx(pme->m_pSKMenu, &pai);

	IMENUCTL_GetRect(pme->m_pMenu, &rc);
	IMENUCTL_GetRect(pme->m_pSKMenu, &rcs);

	// See if the main menu changed...

	ccy = cy - rcs.dy;
	if(rc.dy != ccy)
	{
		rc.dy = ccy;
   		IMENUCTL_SetRect(pme->m_pMenu, &rc);
	}
	
	IMENUCTL_Redraw(pme->m_pSKMenu);
	IMENUCTL_Redraw(pme->m_pMenu);
	
	// Activate the icon view menu
	IMENUCTL_SetActive(pme->m_pMenu, TRUE);
	IMENUCTL_SetActive(pme->m_pSKMenu, TRUE);

	IDISPLAY_Update(pme->a.m_pIDisplay); 

}

static void DevelopKamja_Free(CDevelopKamja* pme)
{
	if(pme->m_pKamja)
		IIMAGE_Release(pme->m_pKamja);
	if(pme->m_pJahyen)
		IIMAGE_Release(pme->m_pJahyen);
	if(pme->m_pMenu){
		IMENUCTL_Release(pme->m_pMenu);
		pme->m_pMenu = NULL;
	}
	if(pme->m_pSKMenu){
		IMENUCTL_Release(pme->m_pSKMenu);
		pme->m_pSKMenu = NULL;
	}
	
}
