/*===========================================================================

FILE: HtmlSample.c
===========================================================================*/


/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"		// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services
#include "AEEStdLib.h"		// AEE StdLib Services
#include "AEEAppGen.h"		// AEEApplet structure and Applet services
#include "AEEFile.h"		// IFILEMGR and IFILE
#include "AEEMenu.h"		// AEE MenuCtl Services
#include "AEEWeb.h"			// AEE Web Services
#include "AEEHtmlViewer.h"	// IHtmlViewer interface
// Services used by app
#include "AEENet.h"		//	AEENet Services

#include "HtmlSample.bid" 

#define MENU_EVT_HTMLVIEW	401
#define MENU_EVT_FILEOPEN	402
#define MENU_EVT_CONNECT	403

#define INFLATERECT(prc,nx,ny) \
   (prc)->x-=(nx),(prc)->y-=(ny),(prc)->dx+=((nx)*2),(prc)->dy+=((ny)*2)

typedef struct _CIHtmlSample
{
	AEEApplet		a;					// Mandatory first AEEApplet data member
	IMenuCtl*		m_pIMenu;
	IWeb*			m_pIWeb;			// IWeb interface ptr
	IWebResp*		m_pIWebResp;		// IWebResp interface ptr
	IHtmlViewer *	m_pHtmlViewer;
	AEECallback		m_Callback;
	char*			m_BodyBuffer;		// Buffer to hold HTML data
	int				m_BodySize;			// Size of HTML data in buffer
	int				m_BodyAllocSize;	// Allocated size of HTML data buffer
	int             m_nLineHeight;
	AEEDeviceInfo   m_dInfo;
	char*			m_url;
} CIHtmlSample;

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static void DisplayMenu(CIHtmlSample *pMe);
static boolean HtmlSample_InitAppData(IApplet* pi);
static void HtmlSample_FreeAppData(IApplet* pi);
static void DisplayOutput(IApplet * pMe, int nline, char *pszStr, RGBVAL clrColor);
void DisconnectWebServer(CIHtmlSample * pMe);
static void WebReadCB(void* cxt);
static void WebStatusNotification(void* pNotifyData, WebStatus ws, void* p);
static void HViewerApp_NotifyCB(CIHtmlSample *pMe, HViewNotify *phvn);
void DrawIHtmlViewer(CIHtmlSample *pMe);
static boolean HtmlSample_HandleEvent(IApplet * pi, AEEEvent eCode, 
                                      uint16 wParam, uint32 dwParam);

/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDisplay_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDisplay_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   if(ClsId == AEECLSID_HTMLSAMPLE){
      if(AEEApplet_New(sizeof(CIHtmlSample), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)HtmlSample_HandleEvent,(PFNFREEAPPDATA)HtmlSample_FreeAppData)
         == TRUE)
      {
		 // Add your code here .....
		 if (HtmlSample_InitAppData((IApplet*)*ppObj) == TRUE)
				return (AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION HtmlSample_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean HtmlSample_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean HtmlSample_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   	CIHtmlSample * pMe = (CIHtmlSample *)pi;
	
	//if (pMe->m_pHtmlViewer && IHTMLVIEWER_HandleEvent(pMe->m_pHtmlViewer, eCode, wParam, dwParam))
	//	return TRUE;
	if(pMe->m_pIMenu)
	{
		if(IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE)
		{
			if (pMe->m_pHtmlViewer && IHTMLVIEWER_HandleEvent(pMe->m_pHtmlViewer, eCode, wParam, dwParam))
				return TRUE;
		}
	}
	switch (eCode) 
	{
		case EVT_APP_START:                        
		    
		    // Add your code here .....
			{
				DisplayMenu(pMe);
				return TRUE;
			}
		case EVT_KEY:
			if (pMe->m_pIMenu)
			{
				if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) && (wParam == AVK_RIGHT))
				{
					DisconnectWebServer(pMe);
					if(pMe->m_pHtmlViewer)
					{
						IHTMLVIEWER_Release(pMe->m_pHtmlViewer);
						pMe->m_pHtmlViewer = NULL;
					}		
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw(pMe->m_pIMenu);
				}
			    return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
			}
			//else 
			//	return FALSE;
			if (pMe->m_pHtmlViewer)
				return IHTMLVIEWER_HandleEvent(pMe->m_pHtmlViewer, eCode, wParam, dwParam);
			else
				return FALSE;
		case EVT_APP_STOP:
		    // Add your code here .....
			return TRUE;
		case EVT_COMMAND:
			{
				switch(wParam)
				{
				case MENU_EVT_HTMLVIEW:
					{
						const char testHtml[] = (
							"<a href='http://littlekamja.gaeasoft.co.kr/kamja.asp'></a><br>"
							"<form action='http://www.yahoo.com'><br>"
							"HTML  <br>"
							"<input type=text name=TextCtl value='ؽƮƮ' size=50><br>"
							"üũڽ<br>"
							"<input type=checkbox name=ChkBox value=Option1 checked ><br>"
							"<input type=checkbox name=ChkBox value=Option2 ><br>"
							"<input type=submit><br>"
							"</form><br>"
							"<h1 align=left> </h1><br>"
							"<p align=center>߰ </p><br>"
							"<h1 align=right> </h1><br>"
 							"<font color=maroon>ؽƮ ÷</font><br>"
							"<b> </b><br>"	
							"<big>ū </big><br>"
							"<p>END</p>"
							"\0"
							);
						
						AEERect rc;
						
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						
						//SETAEERECT(&originalRC, 0,0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);
						//INFLATERECT(&originalRC, -4, -4);
						
						SETAEERECT(&rc, 0,0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);
						IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rc, MAKE_RGB(0xc0, 0xc0, 0xc0));
						
						INFLATERECT(&rc, -4, -4);
						IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rc, RGB_BLACK);
						
						INFLATERECT(&rc, -1, -1);
						IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rc, RGB_WHITE);

						if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_HTML, (void**)&pMe->m_pHtmlViewer) != SUCCESS)
						{
							DisplayOutput((IApplet *)pMe, 4, "IHtmlView  ", MAKE_RGB(0,0,0));
							DisconnectWebServer(pMe);
							return TRUE;
						}
						IHTMLVIEWER_SetNotifyFn(pMe->m_pHtmlViewer, (PFNHVIEWNOTIFY)HViewerApp_NotifyCB, pMe);
						IHTMLVIEWER_SetRect(pMe->m_pHtmlViewer, &rc);
						IHTMLVIEWER_SetData(pMe->m_pHtmlViewer, testHtml, STRLEN(testHtml));
						//IHTMLVIEWER_ParseBuffer(pMe->m_pHtmlViewer, testHtml);
						return TRUE;
					}
				case MENU_EVT_FILEOPEN:
					{
						IFileMgr * pifm;
						IFile *pif;
						
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						
						if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_HTML, (void**)&pMe->m_pHtmlViewer) != SUCCESS)
						{
							DisplayOutput((IApplet *)pMe, 4, "IHtmlView  ", MAKE_RGB(0,0,0));
							return TRUE;
						}
						
						IHTMLVIEWER_SetNotifyFn(pMe->m_pHtmlViewer, (PFNHVIEWNOTIFY)HViewerApp_NotifyCB, pMe);
						DrawIHtmlViewer(pMe);
						
						if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void**)&pifm) != SUCCESS) 
						{
							DisplayOutput((IApplet *)pMe, 4, "IFileMgr  ", MAKE_RGB(0,0,0));
							return TRUE;
						}
						
						pif = IFILEMGR_OpenFile(pifm, "kamja.htm", _OFM_READ);
               
						if (pif) 
						{
							IHTMLVIEWER_LoadStream(pMe->m_pHtmlViewer, (IAStream*)pif);
							IFILE_Release(pif);
						}
						IFILEMGR_Release(pifm);
						pifm = NULL;
						//else {
						// cannot find "htmltest.htm"
						//IHTMLVIEWER_ParseBuffer(pme->pIHtmlViewer, gszError);
						//}
						return TRUE;
					}
				case MENU_EVT_CONNECT:
					{
						const char *hostName = "http://littlekamja.gaeasoft.co.kr/kamja.htm";
						
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
				

						if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_HTML, (void**)&pMe->m_pHtmlViewer) != SUCCESS)
						{
							DisplayOutput((IApplet *)pMe, 4, "IHtmlView  ", MAKE_RGB(0,0,0));
							return TRUE;
						}
						
						IHTMLVIEWER_SetNotifyFn(pMe->m_pHtmlViewer, (PFNHVIEWNOTIFY)HViewerApp_NotifyCB, pMe);
						DrawIHtmlViewer(pMe);
						
						if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_WEB, (void**)(&pMe->m_pIWeb)) != SUCCESS) 
						{
							DisplayOutput((IApplet *)pMe, 1, "IDS_ERR_IWEB_FAILED", MAKE_RGB(0xff, 0, 0));
							DisconnectWebServer(pMe);
							return TRUE;
						}			

						CALLBACK_Init(&pMe->m_Callback, WebReadCB, pMe);
						// Note the odd syntax of this call, which is necessary because it's a macro that
						// gets expanded before the actual variable arguments function call is made.
						IWEB_GetResponse(pMe->m_pIWeb, (pMe->m_pIWeb, &pMe->m_pIWebResp, &pMe->m_Callback, hostName,
										WEBOPT_HANDLERDATA, pMe,
										WEBOPT_STATUSHANDLER, WebStatusNotification,
										WEBOPT_END));
						return TRUE;
					}
				default:
					return TRUE;
				}
			}
		default:
			break;
	}
	return FALSE;
}

static void WebStatusNotification(void* pNotifyData, WebStatus ws, void* p)
{
	CIHtmlSample* pMe = (CIHtmlSample*)pNotifyData;
	switch (ws)
	{
	case WEBS_CANCELLED:
		DBGPRINTF("IDS_WEBS_CANCELLED");
		break;
	case WEBS_GETHOSTBYNAME:
		DBGPRINTF("IDS_WEBS_GETHOSTBYNAME");
		break;		  
	case WEBS_CONNECT:
		DBGPRINTF("IDS_WEBS_CONNECT");
		break;
	case WEBS_SENDREQUEST:
		DBGPRINTF("IDS_WEBS_SENDREQUEST");
		break;
	case WEBS_READRESPONSE:
		DBGPRINTF("IDS_WEBS_READRESPONSE");
		break;
	case WEBS_GOTREDIRECT:
		DBGPRINTF("IDS_WEBS_GOTREDIRECT");
		break;
	case WEBS_CACHEHIT:
		DBGPRINTF("IDS_WEBS_CACHEHIT");
		break;
	}
}


static void WebReadCB(void* cxt)
{
	CIHtmlSample* pMe = (CIHtmlSample*)cxt;
	WebRespInfo* pWebRespInfo = IWEBRESP_GetInfo(pMe->m_pIWebResp);
	int ByteCount;
	char buf[256];
	SPRINTF(buf, "Error Code %d", pWebRespInfo->nCode);
	DBGPRINTF(buf);
	if (pWebRespInfo && WEB_ERROR_SUCCEEDED(pWebRespInfo->nCode))
	{
		//ISource* pISource = pWebRespInfo->pisMessage;
		//DBGPRINTF("WebReadCB");
		IHTMLVIEWER_LoadSource(pMe->m_pHtmlViewer, pWebRespInfo->pisMessage);
		//ISOURCE_Release(pISource);
		/*
		ByteCount = ISOURCE_Read(pISource, (char*)buf, sizeof(buf));
		//DisplayOutput((IApplet *)pMe, 2, "WebReadCB", MAKE_RGB(0,0,0));
		// Read() return values fall into four categories of interest:
		//
		//   1. ISOURCE_WAIT:  This means data is not yet available
		//   2. ISOURCE_ERROR: An eror occurred
		//   3. ISOURCE_END:   No more data
		//   4. >0 : The actual data that was read
		//

		switch (ByteCount)
		{
		case ISOURCE_WAIT:
			//xStatus(pMe, 1, IDS_WAITING);
			ISOURCE_Readable(pISource, &pMe->m_Callback);
			return;

		case ISOURCE_ERROR:
			// Connection hosed.
			//xStatus(pMe, 1, IDS_ERR_READ);
			//StopProgressDisplay(pMe);
			//pMe->m_bDownloadErr = TRUE;
			break;

		case ISOURCE_END:
			// No more data, display the results
			if (pMe->m_BodySize < pMe->m_BodyAllocSize)
				pMe->m_BodyBuffer[pMe->m_BodySize] = 0;
			else
				pMe->m_BodyBuffer[pMe->m_BodyAllocSize - 1] = 0;
			
			//DisplayResults(pMe);
			DBGPRINTF(pMe->m_BodyBuffer);
			//IHTMLVIEWER_ParseBuffer(pMe->m_pHtmlViewer, pMe->m_BodyBuffer);
			IHTMLVIEWER_LoadSource(pMe->m_pHtmlViewer, pISource);
			break;

		default:
			//xStatus(pMe, 1, IDS_DOWNLOADING);
			if (ByteCount)
				if (pMe->m_BodySize + ByteCount > pMe->m_BodyAllocSize)
				{
			{
					const int NewSize = pMe->m_BodyAllocSize + 1024;
					char* NewBuf = (char*)REALLOC(pMe->m_BodyBuffer, NewSize);

					if (NewBuf)
					{
						pMe->m_BodyBuffer = NewBuf;
						pMe->m_BodyAllocSize = NewSize;
					}
				}
				if (pMe->m_BodySize + ByteCount <= pMe->m_BodyAllocSize)
				{
					MEMCPY(pMe->m_BodyBuffer + pMe->m_BodySize, buf, ByteCount);
					pMe->m_BodySize += ByteCount;
				}
			}

			ISOURCE_Readable(pISource, &pMe->m_Callback);
			return;
		}
		*/
	}
	DisconnectWebServer(pMe);

	/*
	if(pWebRespInfo->nCode == 302)
	{
		if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_WEB, (void**)(&pMe->m_pIWeb)) != SUCCESS) 
		{
			DisplayOutput((IApplet *)pMe, 1, "IDS_ERR_IWEB_FAILED", MAKE_RGB(0xff, 0, 0));
			DisconnectWebServer(pMe);
			return;
		}
		DBGPRINTF("URL");			
		DBGPRINTF(pMe->m_url);
		DBGPRINTF("URL");			
		
		CALLBACK_Init(&pMe->m_Callback, WebReadCB, pMe);
		// Note the odd syntax of this call, which is necessary because it's a macro that
		// gets expanded before the actual variable arguments function call is made.
		IWEB_GetResponse(pMe->m_pIWeb, (pMe->m_pIWeb, &pMe->m_pIWebResp, &pMe->m_Callback, pMe->m_url,
									WEBOPT_HANDLERDATA, pMe,
									WEBOPT_STATUSHANDLER, WebStatusNotification,
									WEBOPT_END));
	}
	*/
}
static void DisplayMenu(CIHtmlSample *pMe)
{
	AEERect rc;
	AECHAR  szBuf[50];
	AEEMenuColors mclr;
	
	if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu) != SUCCESS)
		return;
	
	// Set display rectangle
	SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

	// õ ޴  Ѵ. 
	mclr.wMask = (MC_BACK | MC_TEXT | MC_SEL_BACK | MC_SEL_TEXT | MC_FRAME);			
	mclr.cBack = RGB_WHITE;
	mclr.cText = RGB_BLACK;
	mclr.cSelBack = MAKE_RGB (146, 109, 255);
	mclr.cSelText = RGB_BLACK;
	mclr.cFrame = RGB_WHITE;

	IMENUCTL_SetColors (pMe->m_pIMenu, &mclr);
		
	// ޴ Ʈ ŸƲ Ѵ
	STREXPAND((byte *)"", STRLEN(""), szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);
	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  
	
	STREXPAND((byte *)"1. HTML  ", STRLEN("1. HTML  "), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_HTMLVIEW, szBuf, 0);
	
	STREXPAND((byte *)"2. HTML  о", STRLEN("2. HTML  о"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_FILEOPEN, szBuf, 0);

	STREXPAND((byte *)"3.  HTML  о", STRLEN("3.  HTML  о"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_CONNECT, szBuf, 0);

	
	//STREXPAND((byte *)"2. GetHostByName ̿ϱ", STRLEN("2. GetHostByName ̿ϱ"), szBuf, sizeof(szBuf));
	//IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_GETHOST, szBuf, 0);

	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

static boolean HtmlSample_InitAppData(IApplet* pi)
{
	CIHtmlSample* pMe = (CIHtmlSample*)pi;
	int pnAscent = 0;
	int pnDescent = 0;

	pMe->m_pIMenu = NULL;
	pMe->m_pIWeb = NULL;
	pMe->m_pIWebResp = NULL;
	pMe->m_BodyBuffer = NULL;
	pMe->m_url = NULL;
	pMe->m_pHtmlViewer = NULL;
	pMe->m_BodySize = 0;
	pMe->m_BodyAllocSize = 0;

	pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);
	
	/*  ڿ ߰ */
	/*
	if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_HTML, (void**)&pMe->m_pHtmlViewer) != SUCCESS)
	{
		DisplayOutput((IApplet *)pMe, 4, "HTML CREATE Error", MAKE_RGB(0,0,0));
		DisconnectWebServer(pMe);
		return TRUE;
	}
	IHTMLVIEWER_SetNotifyFn(pMe->m_pHtmlViewer, (PFNHVIEWNOTIFY)HViewerApp_NotifyCB, pMe);
	*/
	return TRUE;

}
static void HtmlSample_FreeAppData(IApplet* pi)
{
	CIHtmlSample* pMe = (CIHtmlSample*)pi;
	if (pMe->m_pIMenu != NULL)
	{
		// ޴ Ʈ ϴ Լ
		IMENUCTL_Release (pMe->m_pIMenu);
		pMe->m_pIMenu = NULL;
	}
	DisconnectWebServer(pMe);
	
	if(pMe->m_pHtmlViewer)
	{
		IHTMLVIEWER_Release(pMe->m_pHtmlViewer);
		pMe->m_pHtmlViewer = NULL;
	}
    /*
	if(pMe->m_url)
	{
		FREE(pMe->m_url);
		pMe->m_url = NULL;
	}
	*/
}

void DisconnectWebServer(CIHtmlSample * pMe)
{	
	CALLBACK_Cancel(&pMe->m_Callback);
    if (pMe->m_pIWebResp)
	{
		IWEBRESP_Release(pMe->m_pIWebResp);
		pMe->m_pIWebResp = NULL;
	}

	if (pMe->m_pIWeb) 
	{
		IWEB_Release(pMe->m_pIWeb);
		pMe->m_pIWeb = NULL;
	}

	/*
	if (pMe->m_BodyBuffer)
	{
		FREE(pMe->m_BodyBuffer);
		pMe->m_BodyBuffer = NULL;
	}
	*/
	//if(pMe->m_url)
	//{
	//	FREE(pMe->m_url);
	//	pMe->m_url = NULL;
	//}
	//if(pMe->m_pHtmlViewer)
	//{
	//	IHTMLVIEWER_Release(pMe->m_pHtmlViewer);
	//	pMe->m_pHtmlViewer = NULL;
	//}
	pMe->m_BodySize = 0;
	pMe->m_BodyAllocSize = 0;
}

static void DisplayOutput(IApplet * pi, int nline, char *pszStr, RGBVAL clrColor)
{
   AEEDeviceInfo di;        // Device Info
   AECHAR szBuf[200] = {0}; // a buffer that supports 200 char string
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
	AEERect rc;

   AEEApplet * pMe = (AEEApplet*)pi;

   if (pMe == NULL)
      return;
	   
   // Get device information
   ISHELL_GetDeviceInfo(pMe->m_pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pMe->m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   // STR_TO_WSTR ((char *)pszStr, szBuf, sizeof (szBuf));
   
   STREXPAND((byte *)pszStr, STRLEN(pszStr), szBuf, sizeof(szBuf));
   
   //STREXPAND("ISHELL ", sizeof("ISHELL "), szBuf, sizeof(szBuf));
   // If nlines is less than zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
         return;

		SETAEERECT(&rc, 0, dy, di.cxScreen, charHeight);
		IDISPLAY_EraseRect(pMe->m_pIDisplay, &rc);
		IDISPLAY_SetColor(pMe->m_pIDisplay,CLR_USER_TEXT,clrColor);
        IDISPLAY_DrawText(pMe->m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line. 
      IDISPLAY_Update(pMe->m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }
   return;   
}

/*
static void HViewerApp_NotifyCB(CIHtmlSample *pme, HViewNotify *phvn)
{

	char buf[128];
	const char *url = "http://littlekamja.gaeasoft.co.kr/form";
	const char *temp;
	int strLen = 0;
	SPRINTF(buf, "pnvn->code %d", phvn->code);
	switch (phvn->code) {
		case HVN_REDRAW_SCREEN:
			IDISPLAY_ClearScreen(pme->a.m_pIDisplay);
			//if (pme->bViewerShown) {
			//   HViewerApp_FrameViewer(pme);
				IHTMLVIEWER_Redraw(pme->m_pHtmlViewer);
			//} else {
			//   HViewerApp_DrawPict(pme);
			//}
			DBGPRINTF("HVN_REDRAW_SCREEN!");
			break;
		case HVN_JUMP:
			//if (phvn->u.jump.pszURL && STRNICMP(phvn->u.jump.pszURL, "test:", 5) == 0) {
			//   HViewerApp_GoTo(pme, ATOI(phvn->u.jump.pszURL+5));
			//}
		{
			char* NewBuf = (char*)REALLOC(pme->m_url, 100);
			
			if (NewBuf)
			{
				pme->m_url = NewBuf;
			}
			
			MEMCPY(pme->m_url, phvn->u.jump.pszURL, 100);

			if (ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_WEB, (void**)(&pme->m_pIWeb)) != SUCCESS) 
			{
				DisplayOutput((IApplet *)pme, 1, "IDS_ERR_IWEB_FAILED", MAKE_RGB(0xff, 0, 0));
				DisconnectWebServer(pme);
				return;
			}
						
			//CALLBACK_Cancel(&pme->m_Callback);
			
			CALLBACK_Init(&pme->m_Callback, WebReadCB, pme);
			// Note the odd syntax of this call, which is necessary because it's a macro that
			// gets expanded before the actual variable arguments function call is made.
			IWEB_GetResponse(pme->m_pIWeb, (pme->m_pIWeb, &pme->m_pIWebResp, &pme->m_Callback, phvn->u.jump.pszURL,
										WEBOPT_HANDLERDATA, pme,
										WEBOPT_STATUSHANDLER, WebStatusNotification,
										WEBOPT_END));
			DBGPRINTF("pme->m_url");
			DBGPRINTF(pme->m_url);
			DBGPRINTF("pme->m_url");
		}
			break;
		case HVN_SUBMIT:
			// See phvn->u.submit for information on form submission (URL, method, etc.)
			DBGPRINTF("Submitted!");
			break;
		case HVN_DONE:
			DBGPRINTF("Submitted!");
			break;
		case HVN_FOCUS:
			DBGPRINTF("HVN_FOCUS!");
			break;
		default:
			break;
   }
   DBGPRINTF(buf);

}
*/

static void HViewerApp_NotifyCB(CIHtmlSample *pMe, HViewNotify *phvn)
{
	switch (phvn->code) {
/* ؽƮ Է  Ǯũ 带 ϰ ȴ.   Է ϱ  Ǯ ũ 带    ȭ鿡 IHtmlViewer  ٽ ׷ϴ    ߻ϴ ̺Ʈ HVN_REDRAW_SCREEN̴. */
		case HVN_REDRAW_SCREEN:
			{
			/*	AEERect rc;
			IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
			
			SETAEERECT(&rc, 0,0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);
			IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rc, MAKE_RGB(0xc0, 0xc0, 0xc0));
						
			INFLATERECT(&rc, -4, -4);
			IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rc, RGB_BLACK);
						
			INFLATERECT(&rc, -1, -1);
			IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rc, RGB_WHITE);
			IHTMLVIEWER_SetRect(pMe->m_pHtmlViewer, &rc);
			*/
			DrawIHtmlViewer(pMe);
			DBGPRINTF("HVN_REDRAW_SCREEN");
			break;
			}
		/* ؽƮ Ǯũ 带   ߻ϴ ̺Ʈ̴. */
		case HVN_FULLSCREEN_EDIT:
			DBGPRINTF("HVN_FULLSCREEN_EDIT");
			break;
		/* ۸ũ   ߻ȴ. */
		case HVN_JUMP:
			DBGPRINTF("HVN_JUMP");
			break;
		/* Submit ư Ŭ  ߻ȴ. */
		case HVN_SUBMIT:
			DBGPRINTF("HVN_SUBMIT");
			break;
		/* HTML ͸ д ۾   ߻ȴ. */ 
		case HVN_DONE:
			DBGPRINTF("HVN_DONE");
			break;
		/* ٸ HTML  Ŀ ̵  ߻ȴ. */
		case HVN_FOCUS:
			DBGPRINTF("HVN_FOCUS");
			break;
		default:
			break;
   }
}

void DrawIHtmlViewer(CIHtmlSample *pMe)
{
	AEERect rc;
	IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
			
	SETAEERECT(&rc, 0,0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);
	IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rc, MAKE_RGB(0xc0, 0xc0, 0xc0));
						
	INFLATERECT(&rc, -4, -4);
	IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rc, RGB_BLACK);
						
	INFLATERECT(&rc, -1, -1);
	IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rc, RGB_WHITE);
	IHTMLVIEWER_SetRect(pMe->m_pHtmlViewer, &rc);
}
