/*===========================================================================

FILE: SoundSample.c
===========================================================================*/


/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"          // Module interface definitions
#include "AEEAppGen.h"          // Applet interface definitions
#include "AEEShell.h"           // Shell interface definitions
#include "AEESound.h"			// Sound Interface definitions

#include "AEEStdLib.h"
#include "AEEMenu.h"
#include "AEEDisp.h"

#include "SoundSample.bid"

#define		MENU_EVT_PLAYTONE		400
#define		MENU_EVT_PLAYTONELIST	401
#define		MENU_EVT_GETDEVICE		402
#define		MENU_EVT_VIBRATE		403
#define		MENU_EVT_VOLUME			404
#define		MENU_EVT_SOUNDPLAYER	405

typedef enum
{
   SOUND_NONE,
   SOUND_VOLUMEUP,
   SOUND_VOLUMEDOWN,
   SOUND_VOLUMEGET
} SoundProcess;

typedef struct _CISoundSample {
   AEEApplet	   a;
   IMenuCtl *	   m_pIMenu;
   ISound *        m_pISound;
   ISoundPlayer *  m_pISoundPlayer;

   AEESoundToneData *   m_pToneDataList; // Tone-list

   AEERect         m_rc;
   int             m_nLineHeight;
   AEEDeviceInfo   m_dInfo;
   uint16		   wVolume;
   SoundProcess	   m_eSoundProcess;
} CISoundSample;

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static boolean SoundSample_HandleEvent(IApplet * pi, AEEEvent eCode, 
                                      uint16 wParam, uint32 dwParam);

static void DisplayMenu(CISoundSample *pMe);
static boolean SoundSample_InitAppData(IApplet* pi);
static void SoundSample_FreeAppData(IApplet* pi);
static void DisplayOutput(IApplet * pMe, int nline, char *pszStr, RGBVAL clrColor);
static void SoundSampleCBFn( void * pUser, AEESoundCmd eCBType, AEESoundStatus eStatus, uint32 dwParam );
static void SoundPlayerCBFn( void * pUser, AEESoundPlayerCmd eCBType, AEESoundPlayerStatus eStatus, uint32 dwParam );

/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDisplay_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDisplay_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   if(ClsId == AEECLSID_SOUNDSAMPLE){
      if(AEEApplet_New(sizeof(CISoundSample), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)SoundSample_HandleEvent,(PFNFREEAPPDATA)SoundSample_FreeAppData)
         == TRUE)
      {
		 // Add your code here .....
		 if (SoundSample_InitAppData((IApplet*)*ppObj) == TRUE)
			return (AEE_SUCCESS);
         
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION SoundSample_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean SoundSample_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean SoundSample_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
	CISoundSample * pMe = (CISoundSample *)pi;
	switch (eCode) 
	{
      case EVT_APP_START:                        
		    
		    // Add your code here .....
			DisplayMenu(pMe);
      		return(TRUE);
      case EVT_KEY:
			if (pMe->m_pIMenu)
			{
				if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) && (wParam == AVK_RIGHT))
				{
					// ISound Interface 
					if(pMe->m_pISound != NULL)
					{
						ISOUND_RegisterNotify(pMe->m_pISound, NULL, NULL);
						// Stop tone if any are playing
						if ( pMe->m_pToneDataList != NULL )
						{
							FREE(pMe->m_pToneDataList);
							pMe->m_pToneDataList = NULL;
						}
						ISOUND_StopTone (pMe->m_pISound);
						ISOUND_StopVibrate (pMe->m_pISound);
						ISOUND_Release (pMe->m_pISound);
						pMe->m_pISound = NULL;
					}
					pMe->m_eSoundProcess = SOUND_NONE;
					// ISoundPlayer Interface
					if (pMe->m_pISoundPlayer != NULL)
					{
						// First deregister the notify. This way no callbacks 
						// will be posted for the rest of the ISoundPlayer operations.
						ISOUNDPLAYER_RegisterNotify (pMe->m_pISoundPlayer, NULL, NULL);
						// Stop any songs being played at this time
						ISOUNDPLAYER_Stop (pMe->m_pISoundPlayer);
						ISOUNDPLAYER_Release (pMe->m_pISoundPlayer);
						pMe->m_pISoundPlayer = NULL;
					}

					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw(pMe->m_pIMenu);
				}
				if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) && (wParam == AVK_UP))
				{
					uint16 wVol = pMe->wVolume + 10;
                    pMe->wVolume = wVol > AEE_MAX_VOLUME ? 0 : wVol;
                    pMe->m_eSoundProcess = SOUND_VOLUMEUP;
					ISOUND_SetVolume(pMe->m_pISound, pMe->wVolume);
				}
				if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) && (wParam == AVK_DOWN))
				{
					int nVol = pMe->wVolume - 10;
                    pMe->wVolume = nVol < 0 ? AEE_MAX_VOLUME : (uint16)nVol;
     				pMe->m_eSoundProcess = SOUND_VOLUMEDOWN;
                    ISOUND_SetVolume(pMe->m_pISound, pMe->wVolume);
				}
			    return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
			}
			else 
				return FALSE;
	  case EVT_APP_STOP:

		    // Add your code here .....

         return TRUE;
	  case EVT_COMMAND:
		  {
			  switch(wParam)
			  {
				  case MENU_EVT_PLAYTONE:
					{
						AEESoundToneData toneData;
						
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						DisplayOutput((IApplet *)pMe, 1, " (  Ű )", MAKE_RGB(0,0,0));

						if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound))
							return TRUE; 
						
		                ISOUND_RegisterNotify(pMe->m_pISound, &SoundSampleCBFn, (void *)pMe);

						toneData.eTone = AEE_TONE_0;
                        toneData.wDuration = 2000;     // in milliseconds

						// Start playing tone
						ISOUND_PlayTone (pMe->m_pISound, toneData);

						return TRUE;
					}
				  case MENU_EVT_PLAYTONELIST:
					{
						uint16 wDataLen = 4;
						
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						DisplayOutput((IApplet *)pMe, 1, " (  Ű )", MAKE_RGB(0,0,0));

						if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound))
							return TRUE; 
						// 츮 4   ̴. 
						pMe->m_pToneDataList = MALLOC(sizeof(AEESoundToneData)*wDataLen);
						
						if(pMe->m_pToneDataList != NULL)
						{
							pMe->m_pToneDataList[0].eTone = AEE_TONE_0;
							pMe->m_pToneDataList[0].wDuration = 1000; // In milliseconds

							pMe->m_pToneDataList[1].eTone = AEE_TONE_1;
							pMe->m_pToneDataList[1].wDuration = 2000;

							pMe->m_pToneDataList[2].eTone = AEE_TONE_2;
							pMe->m_pToneDataList[2].wDuration = 3000;

							pMe->m_pToneDataList[3].eTone = AEE_TONE_3;
							pMe->m_pToneDataList[3].wDuration = 4000;

							ISOUND_RegisterNotify(pMe->m_pISound, &SoundSampleCBFn, (void *)pMe);
			                ISOUND_PlayToneList (pMe->m_pISound, pMe->m_pToneDataList, wDataLen);
						}
 						return TRUE;
					}
				  case MENU_EVT_GETDEVICE:
					{
						AEESoundInfo sndInfo = {0};

						// Create ISound interface object. This step needs to be done only
						// if the ISound interface object have not already been created. 
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						DisplayOutput((IApplet *)pMe, 1, " ̽ (  Ű )", MAKE_RGB(0,0,0));
						if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound))
							return TRUE; 
						
						ISOUND_RegisterNotify(pMe->m_pISound, &SoundSampleCBFn, (void *)pMe);

							// ISOUND_Get: gets the device attributes of ISound 
							// interface object.
							ISOUND_Get (pMe->m_pISound, &sndInfo);

							// These are the expected default values that we are 
							// checking against.
							switch(sndInfo.eDevice)
							{
							case AEE_SOUND_DEVICE_UNKNOWN:
								DisplayOutput ((IApplet *)pMe, 4, "Device: UNKNOWN", MAKE_RGB(0xff, 0, 0));
								break;
							case AEE_SOUND_DEVICE_HANDSET:
								DisplayOutput ((IApplet *)pMe, 4, "Device: HANDSET", MAKE_RGB(0xff, 0, 0));
								break;
							case AEE_SOUND_DEVICE_HFK:
								DisplayOutput ((IApplet *)pMe, 4, "Device: HFK", MAKE_RGB(0xff, 0, 0));
								break;
							case AEE_SOUND_DEVICE_HEADSET:
								DisplayOutput ((IApplet *)pMe, 4, "Device: HEADSET", MAKE_RGB(0xff, 0, 0));
								break;
							case AEE_SOUND_DEVICE_SDAC:
								DisplayOutput ((IApplet *)pMe, 4, "Device: SDAC", MAKE_RGB(0xff, 0, 0));
								break;
							case AEE_SOUND_DEVICE_TTY_HFK:
								DisplayOutput ((IApplet *)pMe, 4, "Device: HFK", MAKE_RGB(0xff, 0, 0));
								break;
							case AEE_SOUND_DEVICE_TTY_HEADSET:
								DisplayOutput ((IApplet *)pMe, 4, "Device: Current", MAKE_RGB(0xff, 0, 0));
								break;
							case AEE_SOUND_DEVICE_CURRENT:
								DisplayOutput ((IApplet *)pMe, 4, "Device: Current", MAKE_RGB(0xff, 0, 0));
								break;
							case AEE_SOUND_DEVICE_AHFK:
								DisplayOutput ((IApplet *)pMe, 4, "Device: AHFK", MAKE_RGB(0xff, 0, 0));
								break;
							default:
								DisplayOutput ((IApplet *)pMe, 4, "Device: UNKNOWN", MAKE_RGB(0xff, 0, 0));
								break;
							}
							if (sndInfo.eMethod == AEE_SOUND_METHOD_BEEP)
								DisplayOutput ((IApplet *)pMe, 5, "Method: Beep", MAKE_RGB(0xff, 0, 0));
							if (sndInfo.eAPath == AEE_SOUND_APATH_LOCAL)
								DisplayOutput ((IApplet *)pMe, 6, "APath: Local", MAKE_RGB(0xff, 0, 0));
							if (sndInfo.eEarMuteCtl == AEE_SOUND_MUTECTL_UNMUTED)
								DisplayOutput ((IApplet *)pMe, 7, "Ear MuteCtl: Unmuted", MAKE_RGB(0xff, 0, 0));
							if (sndInfo.eMicMuteCtl == AEE_SOUND_MUTECTL_UNMUTED)
								DisplayOutput ((IApplet *)pMe, 8, "Mic MuteCtl: Unmuted", MAKE_RGB(0xff, 0, 0));

							// Cancel the callback notification
							//ISOUND_RegisterNotify(pISound, NULL, NULL);
							// Release the sound object
							//ISOUND_Release (pISound);
						return TRUE;
					}
				  case MENU_EVT_VIBRATE:
					  {
						  uint16 w16Duration = 2000;      // Time in milliseconds

						  IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						  IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						  IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						  DisplayOutput((IApplet *)pMe, 1, " ߻(  Ű )", MAKE_RGB(0,0,0));
						  if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound))
							return TRUE; 
						
						  ISOUND_Vibrate(pMe->m_pISound, w16Duration);

						  return TRUE;
					  }
				  case MENU_EVT_VOLUME:
					  {
						  AEESoundToneData toneData;
						  IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						  IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						  IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						  DisplayOutput((IApplet *)pMe, 1, " ", MAKE_RGB(0,0,0));
						  
						  if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound))
							return TRUE; 
						
						  ISOUND_RegisterNotify(pMe->m_pISound, &SoundSampleCBFn, (void *)pMe);

						  toneData.eTone = AEE_TONE_RING_A;
                          toneData.wDuration = 4000;     // in milliseconds

						  // Start playing tone
                          //ISOUND_PlayTone (pMe->m_pISound, toneData);

                          // Set the volume to half the maximum volume of the device.
                          // Note: The range of volume is from 0 (lowest) to 
                          //       AEE_MAX_VOLUME (highest).
						  pMe->wVolume = AEE_MAX_VOLUME/2;
						  pMe->m_eSoundProcess = SOUND_VOLUMEUP;
                          ISOUND_SetVolume (pMe->m_pISound, pMe->wVolume);
						  return TRUE;
					  }
				  case MENU_EVT_SOUNDPLAYER:
					  {
			              AEESoundPlayerInfo info;

						  IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						  IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						  IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						  DisplayOutput((IApplet *)pMe, 1, "ISoundPlayer QCELP ", MAKE_RGB(0,0,0));

						  if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void **)&pMe->m_pISoundPlayer))
							  return TRUE; 
						  
						  ISOUNDPLAYER_RegisterNotify(pMe->m_pISoundPlayer, &SoundPlayerCBFn, (void *)pMe);
						  
						  //ISOUNDPLAYER_Set(pMe->m_pISoundPlayer, SDT_FILE, "sample.qcp");
						  info.eInput = SDT_FILE;
						  info.pData = (void *)"sample.qcp";
						  info.dwSize = 0;
						  ISOUNDPLAYER_SetInfo(pMe->m_pISoundPlayer, &info);
						  ISOUNDPLAYER_Play (pMe->m_pISoundPlayer);
						  return TRUE;
					  }
				  default:
					  return TRUE;
			  }
		  }
      default:
         break;
	}
	return FALSE;
}

static void DisplayMenu(CISoundSample *pMe)
{
	AEERect rc;
	AECHAR  szBuf[50];
	AEEMenuColors mclr;
	
	if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu) != SUCCESS)
		return;
	
	// Set display rectangle
	SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

	// õ ޴  Ѵ. 
	mclr.wMask = (MC_BACK | MC_TEXT | MC_SEL_BACK | MC_SEL_TEXT | MC_FRAME);			
	mclr.cBack = RGB_WHITE;
	mclr.cText = RGB_BLACK;
	mclr.cSelBack = MAKE_RGB (146, 109, 255);
	mclr.cSelText = RGB_BLACK;
	mclr.cFrame = RGB_WHITE;

	IMENUCTL_SetColors (pMe->m_pIMenu, &mclr);
		
	// ޴ Ʈ ŸƲ Ѵ
	STREXPAND((byte *)"Sound, SoundPlayer", STRLEN("Sound, SoundPlayer"), szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);
	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  
	
	STREXPAND((byte *)"1. (Tone) ", STRLEN("1. (Tone) "), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_PLAYTONE, szBuf, 0);
	
	STREXPAND((byte *)"2.    ϱ", STRLEN("2.    ϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_PLAYTONELIST, szBuf, 0);

	STREXPAND((byte *)"3.  ̽  ", STRLEN("3.  ̽  "), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_GETDEVICE, szBuf, 0);
	
	STREXPAND((byte *)"4.  ߻Ű", STRLEN("4.  ߻Ű"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_VIBRATE, szBuf, 0);
	
	STREXPAND((char *)"5.  ϱ", STRLEN("5.  ϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_VOLUME, szBuf, 0);
	
	STREXPAND((char *)"6. QCELP   ϱ", STRLEN("6. QCELP   ϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_SOUNDPLAYER, szBuf, 0);
	
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

static boolean SoundSample_InitAppData(IApplet* pi)
{
	CISoundSample* pMe = (CISoundSample*)pi;
	int pnAscent = 0;
	int pnDescent = 0;
	// Initialize the MenuCtl pointer to NULL
	pMe->m_pIMenu = NULL;
	pMe->m_pISound = NULL;
	pMe->m_pISoundPlayer = NULL;
	pMe->m_pToneDataList = NULL;
	pMe->wVolume = AEE_MAX_VOLUME/2;
	pMe->m_eSoundProcess = SOUND_NONE;
	pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);
	return TRUE;

}
static void SoundSample_FreeAppData(IApplet* pi)
{
	CISoundSample* pMe = (CISoundSample*)pi;
	// ޴ Ʈ   ʾҴٸ Ѵ.
	if (pMe->m_pIMenu != NULL)
	{
		// ޴ Ʈ ϴ Լ
		IMENUCTL_Release (pMe->m_pIMenu);
		pMe->m_pIMenu = NULL;
	}
	if (pMe->m_pISound != NULL)
    {
		ISOUND_RegisterNotify(pMe->m_pISound, NULL, NULL);
		ISOUND_StopTone (pMe->m_pISound);
		if ( pMe->m_pToneDataList != NULL )
		{
			FREE(pMe->m_pToneDataList);
			pMe->m_pToneDataList = NULL;
		}
		ISOUND_Release (pMe->m_pISound);
        pMe->m_pISound = NULL;
	}
	// Release SoundPlayer object if it is not released
    if (pMe->m_pISoundPlayer != NULL)
    {
		// Deregister Notify so that 
		ISOUNDPLAYER_RegisterNotify (pMe->m_pISoundPlayer, NULL, NULL);
		// Stop any songs being played at this time
		ISOUNDPLAYER_Stop (pMe->m_pISoundPlayer);
		ISOUNDPLAYER_Release (pMe->m_pISoundPlayer);
		pMe->m_pISoundPlayer = NULL;
	}	

}

static void DisplayOutput(IApplet * pi, int nline, char *pszStr, RGBVAL clrColor)
{
   AEEDeviceInfo di;        // Device Info
   AECHAR szBuf[200] = {0}; // a buffer that supports 200 char string
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
	AEERect rc;

   AEEApplet * pMe = (AEEApplet*)pi;

   if (pMe == NULL)
      return;
	   
   // Get device information
   ISHELL_GetDeviceInfo(pMe->m_pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pMe->m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   // STR_TO_WSTR ((char *)pszStr, szBuf, sizeof (szBuf));
   
   STREXPAND((byte *)pszStr, STRLEN(pszStr), szBuf, sizeof(szBuf));
   
   //STREXPAND("ISHELL ", sizeof("ISHELL "), szBuf, sizeof(szBuf));
   // If nlines is less than zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
         return;

		SETAEERECT(&rc, 0, dy, di.cxScreen, charHeight);
		IDISPLAY_EraseRect(pMe->m_pIDisplay, &rc);
		IDISPLAY_SetColor(pMe->m_pIDisplay,CLR_USER_TEXT,clrColor);
        IDISPLAY_DrawText(pMe->m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line. 
      IDISPLAY_Update(pMe->m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }
   return;   
}

static void SoundSampleCBFn( void * pUser, AEESoundCmd eCBType, AEESoundStatus eStatus, uint32 dwParam )
{
	// ISOUND_RegisterNotify Լ pUser ڿ  ޵ 
	CISoundSample* pMe = (CISoundSample *)pUser;
	char szBuf [30] = {0};
	// dwParam  AEESoundCmdData    
	AEESoundCmdData * pData = (AEESoundCmdData *) dwParam;

	if ( pMe == NULL )
		return;
	//  ޽  ϰ ִ eStatus 
	switch ( eStatus )
	{
	//  ޽  ȭ鿡 ޽ 
		case AEE_SOUND_SUCCESS:
			if(pMe->m_eSoundProcess == SOUND_VOLUMEUP)
			{
				pMe->m_eSoundProcess = SOUND_VOLUMEGET;
				ISOUND_GetVolume(pMe->m_pISound);
			}
			else if(pMe->m_eSoundProcess == SOUND_VOLUMEDOWN)
			{
				pMe->m_eSoundProcess = SOUND_VOLUMEGET;
				ISOUND_GetVolume(pMe->m_pISound);
			}
			else if(pMe->m_eSoundProcess == SOUND_VOLUMEGET)
			{
				if ( pData )
				{
					//  ũ⸦ о´.
					pMe->wVolume = pData->wVolume;
					SPRINTF(szBuf, "  : %u", pMe->wVolume);
					DisplayOutput((IApplet *)pMe, 5, szBuf, MAKE_RGB(0, 0, 0xff));
				}
            
			}
			else
				DisplayOutput ((IApplet *)pMe, 4, "AEE_SOUND_SUCCESS  ޽ ߻ : ", MAKE_RGB(0xff, 0, 0));	
			/*
			if(pMe->m_eSoundProcess == SOUND_VOLUMEUP)
			{
				DisplayOutput ((IApplet *)pMe, 4, " ̱ ", MAKE_RGB(0xff, 0, 0));
				SPRINTF (szBuf, "  : %u", pMe->wVolume);
				DisplayOutput((IApplet *)pMe, 5, szBuf, MAKE_RGB(0, 0, 0xff));

			}
			else if(pMe->m_eSoundProcess == SOUND_VOLUMEDOWN)
			{
				DisplayOutput ((IApplet *)pMe, 4, " ߱ ", MAKE_RGB(0xff, 0, 0));	
				SPRINTF (szBuf, "  : %u", pMe->wVolume);
				DisplayOutput((IApplet *)pMe, 5, szBuf, MAKE_RGB(0, 0, 0xff));
			}
			else
				DisplayOutput ((IApplet *)pMe, 4, "AEE_SOUND_SUCCESS  ޽ ߻ : ", MAKE_RGB(0xff, 0, 0));	
				*/
			break;
		case AEE_SOUND_PLAY_DONE:
			DisplayOutput ((IApplet *)pMe, 4, "AEE_SOUND_PLAY_DONE  ޽ ߻ :   Ϸ", MAKE_RGB(0xff, 0, 0));
			break;
		case AEE_SOUND_FAILURE:
			if(pMe->m_eSoundProcess == SOUND_VOLUMEUP)
				DisplayOutput ((IApplet *)pMe, 4, " ̱ ", MAKE_RGB(0xff, 0, 0));	
			else if(pMe->m_eSoundProcess == SOUND_VOLUMEDOWN)
				DisplayOutput ((IApplet *)pMe, 4, " ߱ ", MAKE_RGB(0xff, 0, 0));	
			else
				DisplayOutput ((IApplet *)pMe, 4, "AEE_SOUND_FAILURE  ޽ ߻ : ", MAKE_RGB(0xff, 0, 0));
			break;
		default:
			DisplayOutput ((IApplet *)pMe, 4, "AEE_SOUND_UNKNOWN  ޽ ߻ :     ޽", MAKE_RGB(0xff, 0, 0));
			break;
	}
}

static void SoundPlayerCBFn( void * pUser, AEESoundPlayerCmd eCBType, AEESoundPlayerStatus eStatus, uint32 dwParam )
{

}